function md_denoise(x,num_knn,lambda,timestep,GraphType,num_iter)
% denoises manifold data via a diffusion process induced by the graph
% Laplacian
% x        : points
% lambda   : reweighting of the weights) with the degree function
%            \tilde{k}_ij = k_ij / pow(d_i d_j, lambda)
% num_knn  : number of k-nearest neighbors
% num_iter : number of iterations
% timestep : explicit=0, implicit=1
% GraphType: Type of graph (0: mutal knn, 1:symmetric knn)

% number of data points and dimension
[dim num] = size(x);

for iter=1:num_iter
    disp(['Iteration:',num2str(iter)]);

    old_x = x;
    [x,KNN,KNNDist] = md_time_step(x,num_knn,lambda,timestep,GraphType);
    diff_x = x - old_x;

    % compute the distance histogram
    [counts,binloc]=hist(reshape(KNNDist,num*num_knn,1),100);

    % clear the matrix of KNN distances
    clear KNNDist;

    % plot all the information
    md_plot_points_dists(x,old_x,diff_x,binloc,counts,iter,lambda,num_knn);

end