function md_denoise_dim_est(x,num_knn,lambda,timestep,GraphType,num_iter)
% Denoises manifold data via a diffusion process induced by the graph
% Laplacian and estimates in each step the dimension of the underlying
% manifold. This information could be used in a stopping criterion.
% Parameters:
%   x        : points
%   lambda   : reweighting of the weights) with the degree function
%              \tilde{k}_ij = k_ij / pow(d_i d_j, lambda)
%   num_knn  : number of k-nearest neighbors
%   num_iter : number of iterations
%   timestep : explicit=0, implicit=1
%   GraphType: Type of graph (0: mutal, 1:symmetric, 2: other symmetric, see
%              getSparseWeightMatrix... .cpp for a better description

% number of data points and dimension
[dim num] = size(x);

for iter=1:num_iter
    disp(['Iteration:',num2str(iter)]);

    old_x = x;
    [x,KNN,KNNDist] = md_time_step(x,num_knn,lambda,timestep,GraphType);
    diff_x = x - old_x;


    %% statistics that are not essential to the denoising algorithm
    % compute the KNN distance histogram
    [countsKNN,binlocKNN]=hist(reshape(KNNDist,num*num_knn,1),100);

    % dimensionality estimation
    dist=getDistanceMatrix(x);
    sort_dist=sort(dist,1);
    min_dist=mean(sort_dist,2);
   
    % estimate intrinsic dimension
    std_min_dist=std(sort_dist(2,:));
    scale=[0; std_min_dist*(1:1:5)'+min_dist(2)];
    dim_histo=histc(reshape(sort_dist(2:end,1:end),(num-1)*num,1),scale);
    ys=log(1/(num*(num-1))*cumsum(dim_histo(1:5)));
    xs=log(scale(2:6));
    dim_est=(length(ys)*sum(ys.*xs)-sum(ys)*sum(xs))/(length(ys)*sum(xs.*xs)-sum(xs)*sum(xs));
    disp(['Estimate of the intrinsic dimension: ',num2str(dim_est)]);

    % estimate continuously the dimension
    [histo,r]=hist(reshape(sort_dist(2:end,1:end),(num-1)*num,1),1000);
    cum=log(1/(num*(num-1))*cumsum(histo));
    desc = zeros(1,995);
    for k=1:995
        ys=cum(k:k+5);
        xs=log(r(k:k+5));
        % slope A of the least squares fit f(x)=Ax+b to the points (xs,ys)
        desc(k) = (length(ys)*sum(ys.*xs)-sum(ys)*sum(xs))/(length(ys)*sum(xs.*xs)-sum(xs)*sum(xs));
    end
    h=sort_dist(num_knn+1,:);
    display(['Average KNN-Dist: ',num2str(mean(h)),'; Standard deviation: ',num2str(std(h)),'; Minimum: ',num2str(min(h)),'; Maximum: ',num2str(max(h))]);
    [counts,binloc]=hist(reshape(sort_dist(2:end,:),(num-1)*num,1),100);
    clear sort_dist;



    % clear the matrix of KNN distances
    clear KNNDist;

    % plot all the information
    md_plot_points_both_dists(x,old_x,diff_x,binlocKNN,countsKNN,binloc,counts,iter,lambda,num_knn);

end