#include <math.h>
#include "mex.h"

void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]) {
  // Test number of parameters.
  if (nrhs != 2 || nlhs != 1) {
    mexWarnMsgTxt("Usage: K = getGammaWeightedMatrix(SparseWeightMatrix, Gamma)");
    return;
  }
  
  // get important parameters
  int num = mxGetN(prhs[0]);
  double gamma = mxGetScalar(prhs[1]);
  int nzmax;
  if(mxIsSparse(prhs[0])) {
    nzmax = mxGetNzmax(prhs[0]);
  } else {
    nzmax = num*num;
  }
    
  // Create output array and compute values
  plhs[0] = mxDuplicateArray(prhs[0]);
  double *out_dists = mxGetPr(plhs[0]);
  for(int i=0; i<nzmax; i++) {
    if(out_dists[i]!=0)
      out_dists[i] = exp(-gamma*out_dists[i]*out_dists[i]);
  }
}
