#include <math.h>
#include "mex.h"

void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]) {
  // Test number of parameters.
  if (nrhs != 4 || nlhs != 1) {
    mexWarnMsgTxt("Usage: K = getSparseWeightMatrix(KNNDist, KNN, radius, GraphType)");
    return;
  }
  double dist,arg1,arg2,Arg,thresh2;
  int k = 0,index=0,i,j;

  // Parse parameters
  int num = (int)mxGetN(prhs[0]);
  int K   = (int)mxGetM(prhs[0]);
  //printf("Number: %i, KNN: %i\n",num,K);

  double* KNNDist = mxGetPr(prhs[0]);
  long*   KNN     =(long*)mxGetPr(prhs[1]);
  double radius = mxGetScalar(prhs[2]);
  double radius_squared = radius*radius;
  int mode  = (int) (mxGetScalar(prhs[3])); // 0: mututal k-nearest neighbor, 1: maximal symmetric k-nearest neighbor, 2:average


  // Allocate memory for output (sparse real matrix)
  int nzmax = 2*num*K; // since we have a symmetric neighborhood graph there can be more than
                           // num*knNum entries in the matrix, 2*num*knNum is an upper bound
                           // typically in high dimensions one has 1.5*num*knNum, however we use
                           // here the upper bound, since otherwise one has to allocate new memory
                           // which is time expensive.


  double* KNNDistMax=new double[num];

  long i_index,j_index;
  for(i=0; i<num; i++)
  {
    i_index=i*K;
    KNNDistMax[i]=0;
    for(j=0;j<K; j++)
    {
      if(KNNDist[i_index+j] > KNNDistMax[i])
       KNNDistMax[i]=KNNDist[i_index+j];
    }
  }

  plhs[0] = mxCreateSparse(num, num, nzmax, mxREAL);
  double* sr = mxGetPr(plhs[0]);
  mwIndex* irs = mxGetIr(plhs[0]);
  mwIndex* jcs = mxGetJc(plhs[0]);

  mexWarnMsgTxt("Starting Generation");
  k=0;
  for(j=0; j<num; j++) {    // loop over columns
    j_index=j*K;
    jcs[j] = k;
    for(i=0; i<K; i++) {     // loop over rows
	// get the distance
	dist = KNNDist[j_index+i];

	// distance relevant?
	arg1 = dist / KNNDistMax[j];
	arg2 = dist / KNNDistMax[KNN[j_index+i]-1];
        switch(mode)
        {
          case 0: Arg = arg1;
	          if(arg2>arg1)
	            Arg = arg2;
                  break;
          case 1: Arg = arg1;
                  if(arg2<arg1)
                    Arg = arg2;
                  break;
          case 2: Arg=0.5*(arg1+arg2); break;
        }

	if(Arg<radius) {
	   if(k>=nzmax) {
	    mexWarnMsgTxt("Allocation of new memory necessary");
	    nzmax += num;
	    mxSetNzmax(plhs[0],nzmax);
	    mxSetPr(plhs[0],(double*)mxRealloc(sr,nzmax*sizeof(double)));
	    mxSetIr(plhs[0],(mwIndex*)mxRealloc(irs,nzmax*sizeof(int)));
	    sr = mxGetPr(plhs[0]);
	    irs = mxGetIr(plhs[0]);
	  }
	  sr[k]  = exp(-Arg*Arg/radius_squared);
	  irs[k] = KNN[j_index+i]-1;
	  k++;
	}
    }
  }
  jcs[num] = k;

  delete KNNDistMax;



  /*k=0;
  for(j=0; j<num; j++) {    // loop over columns
    jcs[j] = k;
    for(m=0;m<counter;m++)
    {
      if(col[m]==j)
      {
        sr[k] =value[m];
        irs[k]=row[m];
        k++;
      }
    }
  }
  jcs[n] = k; */
}