#include "mex.h"
#include <math.h>



void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]) {

  // Test number of parameters.
  if ((nrhs != 1 && nrhs!=2) || nlhs != 1) {
    mexWarnMsgTxt("Usage: g = nn_classify(f, sortInd)");
    return;
  }

  // Parse parameters
  int num = (int)mxGetM(prhs[0]);
  int clCount = (int)mxGetN(prhs[0]);
  int elCount = num*clCount;
  double* inClassif = mxGetPr(prhs[0]);
  double* inIndices = mxGetPr(prhs[1]);

  // lookup table of indices
  int** sortIdx = new int*[num];
  for(int i=0; i<num; i++)
    sortIdx[i] = new int[num];
  for(int i=0; i<num; i++) {
    for(int j=0; j<num; j++) {
      sortIdx[i][j] = (int)(inIndices[i+j*num]+0.1);
    }
  }
  
  
  // Allocate memory for output
  plhs[0] = mxCreateDoubleMatrix(num, clCount, mxREAL);
  double* outClassif = mxGetPr(plhs[0]);

  // Compute Distance Matrix
  for(int i=0; i<elCount; i++) {
    if(!(inClassif[i]==0)) {
      outClassif[i] = inClassif[i];
    } else {
      // get classification of nearest neighbor
      int row = i%num;  // the index of the point
      int col = i/num;
      // find nearest neighbor that is classified
      int nnIdx = 0;
      while((nnIdx<num)&&(inClassif[(sortIdx[nnIdx][row]-1)+num*col]==0))
	nnIdx++;
      if(nnIdx<num)
	outClassif[i] = inClassif[(sortIdx[nnIdx][row]-1)+num*col];
    }
  }
  // delete
  for(int i=0; i<num; i++)
    delete[] sortIdx[i];
  delete sortIdx;
}
